/****************************************************************************************
    Copyright 2004 uja
    Copyright 2004 Trolltech (Qt3)

    This file is part of Edelsteine.
    
    Edelsteine is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
	    
    Edelsteine is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
	    
    You should have received a copy of the GNU General Public License
    along with Edelsteine; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*****************************************************************************************/

// Initialisierung des Spiels:

#include <stdio.h>
#include "chips.h"


chips::chips(QWidget* parent)
{ 
  grenze=3;
  fstart=6;
  zeit=ZEIT0;
  for (int i=0; i<SMAX; i++) 
  { f[i]=i%21;
    h_liste[i]=0;
  }				
  my_timer  =new QTimer(this);
  my_stopper=new QTimer(this); // fuer singleShot
  tic=200;
  
  connect(my_timer,SIGNAL(timeout()),this,SLOT(ticken()));                  // Anzeige alle s
  connect(this,SIGNAL(punkte_changed(int)),parent,SLOT(set_punkte(int)));   // Anzeige Punkte
  connect(this,SIGNAL(zeit_changed(int)),parent,SLOT(set_zeit(int)));       // 10-sStoppuhr
  connect(this,SIGNAL(ticker_changed(int)),parent,SLOT(set_ticker(int)));       // 10-sStoppuhr
  connect(this,SIGNAL(level_changed(QString)),parent,SLOT(set_level(QString))); // Levelanzeige
  my_timer->start(1000);
}

void chips::pause()       { aktiv=false; pausiert=true;  p_pause->show(); }
void chips::weiter()      { aktiv=true;  pausiert=false; p_pause->hide(); }
void chips::game_over()   { aktiv=false; p_gover->show(); emit gameover(punkte); }
void chips::reset_timer() {  zeit=ZEIT0; emit ticker_changed(zeit); emit zeit_changed(zeit); }

// --------- unabhaengiger Kram: -------------

// 10-s-Ticker:
void chips::ticken()
{ 
  if (aktiv)
  { zeit--;
    emit ticker_changed(zeit);
    emit zeit_changed(zeit);
    if (zeit<0) game_over();
  }     
}

// Feldanzeige:
void chips::zeige_feld(int nr) { int i=f[nr]; if ((i>0) && (nr==gewaehlt)) i=i+CMAX; p_sprite[nr]->setFrame(i); }

// Punktegewinn in Abhaengigkeit Kettenlaenge:
int chips::gain(int nr)
{ 
  int sum,i,k=1; 
  if (nr<grenze) sum=0; else { sum=2; for (i=3; i<=nr; i++) { sum=sum+k; k++; } }
  return sum;
}

// Schwierigkeitsgrad in Abhaengigkeit Punkte:
void chips::updat_level()
{
  if (punkte>5000) game_over();
  else if (punkte>2500) { fakt=10; levelname="Guru-Special"; }           // neu
  else if (punkte>1500) { fakt=9; levelname="Experts only"; }            // neu
  else if (punkte>1000) { fakt=8; levelname="Blood, Sweat and Tears"; }  // Gamecraft-Expertenlevel
  else if (punkte>100)  { fakt=7; levelname="Marching on"; }             // Gamecraft-Level
  else { fakt=6; levelname="Rookie"; }                                   // Gamecraft-Uebungslevel
  emit level_changed(QString(levelname));
}

// Kette sammeln, in denen das Glied nr vorkommt,
// zur Kettenliste h_liste addieren: 
void chips::get_kette(int nr)
{
  bool weiter;
  int i,x1=nr%XMAX,y1=(nr-x1)/XMAX,start=nr,ende=nr;
  
  // linken und rechten Endchip bestimmen
  weiter=(x1>0);
  while (weiter) 
  { if (f[start-1]==f[nr]) { x1--; start--; } else weiter=false;
    if (x1<=0) weiter=false;
  }   
  x1=nr%XMAX;
  weiter=(x1<(XMAX-1));
  while (weiter) 
  { if (f[ende+1]==f[nr]) { x1++; ende++; } else weiter=false;
    if (x1>=(XMAX-1)) weiter=false; 
  }
  if ((ende-start)>=(grenze-1)) for (i=start; i<=ende; i++) { h_liste[0]++; h_liste[h_liste[0]]=i; }
   
  // oberen und unteren Endchip bestimmen:
  x1=nr%XMAX; y1=(nr-x1)/XMAX;
  start=y1; ende=y1;
  weiter=(start>0);        
  while(weiter)
  { if (f[nr]==f[x1+(start-1)*XMAX]) start--; else weiter=false;
    if (start<=0) weiter=false;
  }
  y1=ende;
  weiter=(ende<(YMAX-1));
  while(weiter)
  { if (f[nr]==f[x1+(ende+1)*XMAX]) { y1++; ende++; } else weiter=false;
    if (ende>=(YMAX-1)) weiter=false;
  }    
  if ((ende-start)>=(grenze-1)) for (i=start; i<=ende; i++) { h_liste[0]++; h_liste[h_liste[0]]=i*XMAX+x1; }
} 



// -------------------------------------------------------------------------------------------------------
// Befehlszyklus ->kette_auswerten->droppen->fuellen->bereinigen - Globale Variable:wertung
// Einstiege: gefuellte Kettenliste: kette_auswerten, leere Kettenliste:bereinigen
// Ausstieg:  bereinigen
// wertung false: Cleanup-Modus, der Zyklus laeuft durch ohne Darstellung, Punkt und Komma
// wertung true:  Module golgen in halbsekuendlichem Abstand, mit Darstellung und Punkteauswertung  
// --------------------------------------------------------------------------------------------------------

// Punkte sammeln, Schwierigkeitsgrad einstellen, Kette loeschen, weiter an droppen(flag)
void chips::kette_auswerten()
{
  int i;
  if (h_liste[0]>0)
  { aktiv=false;
    for (i=1; i<=h_liste[0]; i++) f[h_liste[i]]=0;
    if (wertung) // Spielmodus 
    {
      for (i=1; i<=h_liste[0]; i++) zeige_feld(h_liste[i]);
      punkte=punkte+gain(h_liste[0]);
      emit punkte_changed(punkte);
      updat_level();
      my_stopper->singleShot(tic,this,SLOT(droppen()));
    }
    else droppen(); // Cleanup-Modus
  }    
} 
  
// Steinluecken beseitigen:
// pro Spalte von unten her besetzte Felder in genullten Vektor einsammeln,
// diesen dann in der Spalte abbilden:
void chips::droppen()
{ 
  int i,j,k,pvek,vek[YMAX];
  for (i=0; i<XMAX; i++)
  { for (j=0; j<YMAX; j++) vek[j]=0;
    pvek=YMAX-1;  // Reserveboden
    for (j=YMAX-1; j>=0; j--) 
    { k=j*XMAX+i;
      if (f[k]>0) { vek[pvek]=f[k]; pvek--; } 
    }
    for (j=0; j<YMAX; j++) { k=i+j*XMAX; f[k]=vek[j]; }
  }
  if (wertung) 
  { 
    for (i=0; i<SMAX; i++) zeige_feld(i); 
    my_stopper->singleShot(tic,this,SLOT(fuellen())); 
  } 
  else fuellen(); 
}  

      
void chips::fuellen()
{ 
  int i;   
  for (i=0; i<SMAX; i++) if (f[i]==0) f[i]=(rand()%fakt)+1; 
  if (wertung) 
  { 
    for (i=0; i<SMAX; i++) zeige_feld(i);
    my_stopper->singleShot(tic,this,SLOT(bereinigen()));
  }
  else bereinigen();
}

// ---------------------------
// beste Kette im Feld suchen, 
// falls keine Kette (mehr) gefunden wird, Verlassen des Befehlszyklus und Rueckgabe Kontrolle an User
void chips::bereinigen() // rekursion auf weiter
{ 
  bool weiter=false;
  int i,max=0,best=-1; 
  // beste Kette bestimmen
  for (i=0; i<SMAX; i++)
  { h_liste[0]=0; 
    get_kette(i);
    if (h_liste[0]<grenze) h_liste[0]=0;
    else if (h_liste[0]>max) { max=h_liste[0]; best=i; } 
  }
  weiter=(best>=0);
  // beste Kette auswerten
  if (weiter)
  { h_liste[0]=0;
    get_kette(best);
    kette_auswerten();
  }
  else aktiv=true;    
}

// -------------------------------------------------------------------------------------------------------------------

// Eingabecheck: ------------------------------------------------------

// Check, ob Zug gueltig ist und Sammeln der zu loeschenden Steine:
bool chips::check_zug(int n1,int n2)
{ 
  bool ok;
  int ddx,ddy,x1=n1%XMAX,y1=(n1-x1)/XMAX,x2=n2%XMAX,y2=(n2-x2)/XMAX;
// Regel 1: Chips muessen Nachbarn sein:
  ddx=x1-x2; ddx=ddx*ddx;
  ddy=y1-y2; ddy=ddy*ddy;
  ok=((ddx+ddy)==1);
// Regel2: 3 gleiche Chips waagerecht oder senkrecht:        
  if (ok)    
  { ddx=0;
    h_liste[0]=0; 
    get_kette(n1);
    if (h_liste[0]<grenze) h_liste[0]=0; else ddx=h_liste[0]; 
    get_kette(n2); 
    if ((h_liste[0]-ddx)<grenze) h_liste[0]=ddx;
    ok=(h_liste[0]>=grenze);
  }    
  return ok;
}

// check, ob noch ein Zug moeglich ist, sonst gameover:
int chips::zug_moeglich()
{ 
  bool ok=false;
  int i,j,k,k1;
  int f0[SMAX];
  // Backup Spielfeld, man kommt ja doch nicht drum herum
  for (i=0; i<SMAX; i++) f0[i]=f[i];
  // jeden Zug einmal durchtesten, bis gefunden:
  for (i=0; i<XMAX-1; i++) for (j=0; j<YMAX-1; j++) if (!ok)
  { k=i+j*XMAX;
    k1=k+1;
    f[k]=f0[k1]; f[k1]=f0[k]; 
    if (check_zug(k,k1)) ok=true; 
    f[k]=f0[k]; f[k1]=f0[k1]; 
    if (!ok) 
    { 
      k1=k+XMAX;
      f[k]=f0[k1]; f[k1]=f0[k]; 
      if (check_zug(k,k1)) ok=true; 
      f[k]=f0[k]; f[k1]=f0[k1];
    }
  }     
  return ok;
}  

// -------------------------------------------------------------------------------------------------------------------
// --- Interaktion mit User abgesehen von dem Kleinvieh ganz oben: ----------------------------------------------------

void chips::neues_spiel()
{ 
  bool losgehts=false;
  int i;
  gewaehlt=-1;
  punkte=0;
  my_stopper->stop();
  fakt=fstart;
  srand(time(0)); 
  emit punkte_changed(punkte); 
  updat_level();
  wertung=false;
  while(!losgehts) // nix GOTO!!!
  { for (i=0; i<SMAX; i++) f[i]=(rand()%fakt)+1;
    bereinigen();
    losgehts=zug_moeglich();
  }
  wertung=true;
  for (i=0; i<SMAX; i++) { p_sprite[i]->show();  zeige_feld(i); }
  p_gover->hide();
  reset_timer();
  weiter();
}

void chips::klick(int nr)
{ 
  if (aktiv)
  {
    int  i,n1;
    if (gewaehlt<0) { gewaehlt=nr; zeige_feld(nr); }
    else if (gewaehlt!=nr)
    { 
      n1=gewaehlt;
      gewaehlt=-1;
      i=f[nr]; f[nr]=f[n1]; f[n1]=i; zeige_feld(n1); zeige_feld(nr);
      if (check_zug(n1,nr)) { reset_timer(); kette_auswerten(); }
      else { i=f[nr]; f[nr]=f[n1]; f[n1]=i;  zeige_feld(n1); zeige_feld(nr); }
    }
  }
  else if (pausiert) weiter();
}
