/****************************************************************************************
    Copyright 2004 uja
    Copyright 2004 Trolltech (Qt3)

    This file is part of Edelsteine.
    
    Edelsteine is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.
	    
    Edelsteine is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
	    
    You should have received a copy of the GNU General Public License
    along with Edelsteine; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*****************************************************************************************/

// Highscore-Routinen Edelsteine

#include "hiscore.h"
#include <stdio.h>
#include <fcntl.h> // getuerktes Lock

void hscrec::setValues(int i,QString datum,QString sname) { punkte=i; zeit=datum; name=sname; }  

void hiscore::hide_hsc() { hide(); }

// ===== Lesen, schreiben, formatieren ===================================================================

QString hiscore::get_feld(QString ein,bool rechts=false,bool fett=false)
{ 
  QString fett_on="<font color=\""; fett_on=fett_on+HICOLOR+"\"><b>";
  QString fett_off="</b></font>";
  QString normal_on="<font color=\""; normal_on=normal_on+LOCOLOR+"\">";
  QString normal_off="</font>";
  QString out="<td ";
  if (rechts) out=out+" align=right";
  out=out+"><nobr>";
  if (fett) out=out+fett_on; else out=out+normal_on; 
  out=out+ein;
  if (fett) out=out+fett_off; else out=out+normal_off;
  out=out+"</nobr></td>";
  return out;
}

void hiscore::dateieinlesen(hscrec my_hscliste[])
{ int i,gelesen=0;
  QString buff,buffint;
  QFile file(HSCFILE);
  if (file.open(IO_ReadOnly))
  { for (i=0; i<MAXEINTRAG; i++)
    { my_hscliste[i].setValues(-999,"---","---"); // Datei anpassen
      gelesen=file.readLine(buff,256);
      if (gelesen>10)
      { buffint=buff.section(';',0,0);
        my_hscliste[i].setValues(buffint.toInt(),buff.section(';',1,1),buff.section(';',2));
      }	 
    }
    file.close();
  }  
}

void dateischreiben(hscrec my_hscliste[])
{ int i,geschrieben=0;
  QString buff;
  QFile file(HSCFILE); // Read the text from a file
  if (file.open(IO_WriteOnly))
  { for (i=0; i<MAXEINTRAG; i++)
    {  buff=buff.setNum(my_hscliste[i].punkte)+';'+my_hscliste[i].zeit+';'+my_hscliste[i].name;
       geschrieben=file.writeBlock(buff,buff.length());
    }
    file.close();
  }  
}

// ===== Block: anzeigen ===========================================================================================
void hiscore::show_hsc()
{ 
  int i,platz=0;
  bool spitze;
  QString buffint,buffout="<br><table>";
  hscrec my_hscliste[MAXEINTRAG];
  QMimeSourceFactory::defaultFactory()->setImage("myimage",QImage(HSCPOTT)); // RichText-Bild kreiert in MimeFactory
  
  dateieinlesen(my_hscliste);
  for (i=0; i<MAXEINTRAG; i++)  
  { platz++;
    spitze=(platz<=HSCGRENZE);
    buffout=buffout+"<tr>";
    buffint=buffint.setNum(platz)+'.';
    if (spitze) buffint="<img src=\"myimage\" align=left> "+buffint;
    buffout=buffout+get_feld(buffint,true,spitze)+get_feld(my_hscliste[i].name,false,spitze);
    buffint=buffint.setNum(my_hscliste[i].punkte,10);
    buffout=buffout+get_feld(buffint,true,spitze)+get_feld(my_hscliste[i].zeit,false,spitze)+"</tr>";
  }  
  buffout=buffout+"</table>"; 

  my_text->setText(buffout);
  my_text->setContentsPos(0,0);
  polish();
  show();
  raise();
}

// ===== Block HSC resetten ==================================================================
void hiscore::reset_hsc()
{ 
  int i,k;
  hscrec my_hscliste[MAXEINTRAG];
  k=QMessageBox::question(this,tr(HSCTITEL),tr(CONFRESET),tr(YES),tr(NO),QString::null,1,1);
  if (k==0)
  { for (i=0; i<MAXEINTRAG; i++) my_hscliste[i].setValues((500-10*i),"--:--","nemo\n");
    dateischreiben(my_hscliste);        
  }
}

// ===== Block HSCSchreiben, Einstieg: save_hsc ================================================
// --- fcntl.h - Bibliothek fuer getuerktes lock: ----------------------------------------------
bool hiscore::get_lock()  { int gotcha=creat("./alles_meins.lck",S_IRWXU); return (gotcha>-1); }
void hiscore::free_lock() { remove("./alles_meins.lck"); }
// ---------------------------------------------------------------------------------------------
void hiscore::eintragen(int punkte,QString datum,QString sname)
{ 
  int i,platz=MAXEINTRAG;
  hscrec my_hscliste[MAXEINTRAG];
  if (get_lock())
  { 
     dateieinlesen(my_hscliste);
     for (i=MAXEINTRAG-1; i>=0; i--) if (punkte>=my_hscliste[i].punkte) platz=i;
     if (platz<MAXEINTRAG)
     { for (i=MAXEINTRAG-1; i>platz; i--) my_hscliste[i]=my_hscliste[i-1];
        my_hscliste[platz].setValues(punkte,datum,sname);
        dateischreiben(my_hscliste);        
     }
     free_lock();
     show_hsc();
  }
  else // try again: 
  { QTimer* my_stopper=new QTimer(this);
    my_stopper->singleShot(rand()%200+50,this,SLOT(eintragen(punkte,datum,sname)));
  }
}
// ----------------------------------------------------------------------------------------------
void hiscore::save_hsc(int punkte)
{ 
  // Punktegrenze bestimmen
  hscrec my_hscliste[MAXEINTRAG];
  dateieinlesen(my_hscliste); 
  if (punkte>=(my_hscliste[MAXEINTRAG-1].punkte)) // Namen holen
  {
    QDateTime timestamp=QDateTime::currentDateTime();
    QString datum=timestamp.toString(TIMEFORM);
    QString blabla,buffint,zeile;
    bool ok;
    buffint=buffint.setNum(punkte,10);
    blabla=HSCMESS0+buffint+HSCMESS1+HSCMESS2;
    QString my_name=QInputDialog::getText(TITEL,blabla,QLineEdit::Normal,QString::null,&ok,0);
    if (ok && !my_name.isEmpty())
    { my_name=my_name.left(32)+"\n"; // Record-Ende einfgen
      eintragen(punkte,datum,my_name);
    }
  }    
}

// ===============================================================================================
hiscore::hiscore(QWidget* parent,char* name): QVBox(parent,name)
{ 
  setPaletteBackgroundColor(BACKGR0);
  setPaletteForegroundColor(COLOR0);
  setMargin(16);
  setSpacing(8);
  setCaption(HSCTITEL);

  QLabel* p_ue=new QLabel(this);
  p_ue->setText(HSCTITELFI);
  
  // Textfeld:
  my_text=new QTextEdit(this);
  my_text->setReadOnly(true);
  my_text->setTextFormat(RichText);
  my_text->setPaletteBackgroundColor(BACKGR2);
  my_text->setPaletteBackgroundPixmap(QPixmap(PAPERBILD));
  my_text->setBackgroundPixmap(QPixmap(BACKBILD2));
  my_text->setFrameShadow(Sunken);
  my_text->setFrameShape(Box);
  my_text->setMargin(8);
  my_text->setMaximumWidth(HSCBREITE);      
  my_text->setMinimumWidth(HSCBREITE);      
  my_text->setMinimumHeight(160);      

  QBrush* my_brush=new QBrush(QColor(BACKGR2),QPixmap(PAPERBILD));
  my_brush->setPixmap(QPixmap("images/birke.png"));
  my_text->setPaper(*my_brush);

  // mittiger Button:
  QHBox* p_mittig=new QHBox(this);
  QLabel* p_leer1=new QLabel(p_mittig);
  QPushButton* p_wech=new QPushButton(SCHLIESSEN,p_mittig);
  QLabel* p_leer2=new QLabel(p_mittig);
  p_mittig->setStretchFactor(p_leer1,10); 
  p_mittig->setStretchFactor(p_leer2,10); 
  p_mittig->setStretchFactor(p_wech,10);
  connect(p_wech,SIGNAL(clicked()),this,SLOT(hide()));         
}
// =========================================================================================================
